import operator

from IPlot import DatasetPlot
from views import DatasetRowPlotView
from compClust.mlx.views import RowSubsetView, ColumnSubsetView
from compClust.mlx.views import SortedView
from compClust.mlx import labelings
from compClust.iplot.mappers import PCADataMapper, RowColorMapper
import compClust.mlx.pcaGinsu

import matplotlib.numerix as nx
try:
  import matplotlib.pylab as pylab
except ImportError, e:
  # try older version of matplotlib matlab(r)-like commands
  import matplotlib.matlab as pylab

# the original pca ginsu name was pcaGinsu, however all of the other plots
# in iplot were of the convention CapWord, so I changed the capitalization
# to make it more consistent
class PCAGinsu(compClust.mlx.pcaGinsu.pcaGinsu):
  """pcaGinsuVisualizeMatplotlib uses pcaGinsu to construct all the necessary labelings
  and then constructs non-interactive matplotlib plots showing the various outlier
  representations.
  """
  # NOTE, if you change the parameter list here, you'll need to change it in the
  # compClust.iplot.IPlot* modules, compClust.mlx.pcaGinsu, and PCAGinsu
  def __init__(self, canvasFactory, dataset, nOutliers=10, sigCutoff = 0.05, 
               maxPCNum = None, verbose = False, rowPCAView = None, makeLabelings = True,
               primaryLabeling=None, secondaryLabeling=None, primaryColumnLabeling=None):
    self.canvasFactory = canvasFactory
    if primaryLabeling is None:
      self.primaryLabeling = dataset.primaryRowLabeling
    else:
      self.primaryLabeling = primaryLabeling
    self.secondaryLabeling = secondaryLabeling

    if primaryColumnLabeling is None:
      self.primaryColumnLabeling = dataset.primaryRowLabeling
    else:
      self.primaryColumnLabeling = primaryColumnLabeling

    compClust.mlx.pcaGinsu.pcaGinsu.__init__(self, dataset, nOutliers, sigCutoff,
                                             maxPCNum, verbose, rowPCAView, makeLabelings)
                                             

  def plotPCNOutlierRowsInOriginalColumnOrder(self, pcNum):
    """
    Return a handle to a new figure that shows a trajectory plot for the
    high and low outliers.  The x-axis is ordered as the columns in the
    dataset are... original column order.
    """
    if self.verbose:
      print "  plotPCNOutlierRowsInOriginalColumnOrder", pcNum
      
    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNum, self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    subset = RowSubsetView(self.dataset, highrows + lowrows)
    dp = self.canvasFactory.getIPlot()
    dp.setTitle("PC %d Outlier Rows In Original Column Order"%(pcNum))
    rv = DatasetRowPlotView(subset, primaryLabeling=self.primaryLabeling, 
                                    secondaryLabeling=self.secondaryLabeling)

    # FIXME: This is kind of ugly, I wish I understood how colors
    # FIXME: were being munged by scaleList better, but it works so
    # FIXME: setting the list is going to be left in.
    rv.getColorMapper().setColors([(0,1,1)]*len(highrows) + [(.6,1,1)]*len(lowrows))

    conditions = self.dataset.primaryColumnLabeling
    if conditions is not None:
      dp.set_xticks_to_labeling(conditions)

    sig_labeling  = self.dataset.getLabeling('PC-%d %d-outlier Up/Flat/Down Columns' \
                                             % (pcNum, self.nOutliers))
    self.format_significant_condition_labels(dp, sig_labeling)
    dp.plot(rv)

    self.dataset.removeView(subset)
    return dp
                

  def plotPCNOutlierRowsInSigGroupOrder(self, pcNum):
    """
    Create and return a matlplotlib figure containing a plot of the
    outlier trajectories across conditions/dimensions where the
    conditions are reordered based (approximately) on significance of
    high vs. low.

    Note: Presently we order conditions/dimensions by mean diff, which is
    approximately ordered by significance of difference, but we SHOULD
    make this more precisely partitioned first into Up/Flat/Down, and
    then within group ordered by mean difference.  This ordering needs
    to correspond to the order of rows output by getOutputForSigGroups.
    """
    if self.verbose:
      print "  plotPCNOutlierRowsInSigGroupOrder", pcNum

    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNum, self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    high_subset = RowSubsetView(self.dataset, highrows)
    high_mean = nx.average(high_subset.getData(),0)
    self.dataset.removeView(high_subset)
    
    low_subset = RowSubsetView(self.dataset, lowrows)
    low_mean = nx.average(low_subset.getData(),0)
    self.dataset.removeView(low_subset)
    
    subset = RowSubsetView(self.dataset, highrows+lowrows)

    mean_differences = high_mean - low_mean
    sorted_subset = SortedView(subset)
    temp = zip(mean_differences, range(len(mean_differences)))
    temp.sort()
    condition_ordering = [x[1] for x in temp]
    sorted_subset.permuteCols(condition_ordering)
    
    #sorted_subset = subset

    dp = self.canvasFactory.getIPlot()
    dp.setTitle("PC %d Outlier Rows In Mean Difference Order"%(pcNum))
    rv = DatasetRowPlotView(sorted_subset,
                            primaryLabeling=self.primaryLabeling, 
                            secondaryLabeling=self.secondaryLabeling)
    
    rv.getColorMapper().setColors([(0,1,1)]*len(highrows) +
                                  [(.6,1,1)]*len(lowrows))

    conditions = self.dataset.primaryColumnLabeling
    if conditions is not None:
      dp.set_xticks_to_labeling(conditions, condition_ordering)

    sig_labeling  = self.dataset.getLabeling('PC-%d %d-outlier Up/Flat/Down Columns' \
                                             % (pcNum, self.nOutliers))
    self.format_significant_condition_labels(dp, sig_labeling, condition_ordering)
    dp.plot(rv)

    subset.removeView(sorted_subset)
    self.dataset.removeView(subset)    
    return dp

  def old_plotPCvsPCWithOutliersInY(self, pcNumForXAxis, pcNumForYAxis):
    """
    Create and return a matplotlib figure containing a scatter plot of
    data points in the dataset when projected onto one principal component
    versus another.  PC numbers passed in need to be 1-origin. 
    """
    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNumForYAxis,
                                                        self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    
    pcaData = self.rowPCAView.getData()
    pylab.plot(pcaData[:,pcNumForXAxis-1],pcaData[:,pcNumForYAxis-1],'b.')

    for highOutlier in highrows:
      pylab.plot([pcaData[highOutlier,pcNumForXAxis-1]],
                 [pcaData[highOutlier,pcNumForYAxis-1]],'ro')

    for lowOutlier in lowrows:
      pylab.plot([pcaData[lowOutlier,pcNumForXAxis-1]],
                 [pcaData[lowOutlier,pcNumForYAxis-1]],'ro')

    pylab.title('PC %d: %d Highest and %d Lowest Outliers' % (pcNumForYAxis,
                                                              self.nOutliers,
                                                              self.nOutliers))
    percentages = self.rowPCAView._RowPCAView__variances * 100;
    pylab.xlabel('PC %d (%4.2f%%)' %(pcNumForXAxis,percentages[pcNumForXAxis-1]))
    pylab.ylabel('PC %d (%4.2f%%)' %(pcNumForYAxis,percentages[pcNumForYAxis-1]))

  def plotPCvsPCWithOutliersInY(self, pcNumForXAxis=0, pcNumForYAxis=1):
    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNumForYAxis,
                                                        self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    variances = self.rowPCAView.getVariances()
    rotMatrix = self.rowPCAView.matrix
    

    # make plotView
    pcaPlot = self.canvasFactory.getDatasetPlot()
    plotView = DatasetRowPlotView(self.rowPCAView, primaryLabeling=self.primaryLabeling, secondaryLabeling=self.secondaryLabeling)
    
    # configure data mapper
    pcaDataMapper = PCADataMapper(plotView)
    pcaDataMapper.setXColumn(pcNumForXAxis)
    pcaDataMapper.setYColumn(pcNumForYAxis)
    plotView.setDataMapper(pcaDataMapper)
    
    # configure color mapper
    def color_outliers(dataset, row, labeling=l):
      label = labeling.getLabelByRow(row)
      if label in ('high', 'low'):
        return 0
      else:
        return .5
    pcaColorMapper = RowColorMapper(plotView)
    pcaColorMapper.setColorByFunction(color_outliers)
    plotView.setColorMapper(pcaColorMapper)
    
    # plot it
    pcaPlot.scatter(plotView)

    # attach legends
    pcaPlot.setTitle("PC %d: %d highest and %d Lowest Outliers"%(pcNumForYAxis,
                                                                 len(highrows),
                                                                 len(lowrows)))
    pcaPlot.axis.set_title('PCA Space', fontdict=pcaPlot.title_font)
    pcaPlot.axis.set_xlabel('PC-%d' %(pcNumForXAxis+1))
    pcaPlot.axis.set_ylabel('PC-%d' %(pcNumForYAxis+1))

    #self.setAxisSquare()
    

  def format_significant_condition_labels(self, dataset_plot, sig_labeling, ordering=None):
    """Given a labeling that defines significance indicate this on the plot
    (also rotate the text so its more of the label is likely to show up on the plot)
    """
    # note: significat_columns is a list of lists
    significant_columns = sig_labeling.getAllColLabels()
    if ordering is not None:
      significant_columns = [ significant_columns[x] for x in ordering ]
    xticklabels = dataset_plot.axis.get_xticklabels()
    for i in range(len(xticklabels)):
      xtick = xticklabels[i]
      xtick.set_rotation(45)
      if significant_columns[i][0] != "flat":
        xtick.set_fontsize('large')
  
  def generateResults(self,rowLabelingNames,colLabelingNames):
    """
    For each principal component, call the major result generating
    functions and save those results to appropriately-named files.
    """
    # Within this function pcNum is 1-origin to match how users would call 
    # the various user-level functions
    for pcNum in range(1,self.maxPCNum+1):

      if self.verbose:
        print '  Writing results for PC %d' % pcNum

      if pcNum > 1:
        f = self.plotPCvsPCWithOutliersInY(pcNum-1, pcNum)
        pylab.savefig('pc%02d-outliers' % pcNum)

      f = self.plotPCNOutlierRowsInSigGroupOrder(pcNum)
      pylab.savefig('pc%02d-outlier-trajectories-order-meandiff' % pcNum)

      output = self.getOutputForPCNOutliers(pcNum,rowLabelingNames)
      write2DStringArrayToFile(output, 'pc%02d-outliers.txt' % pcNum)
      print 'done with PC %d' % (pcNum)

      output = self.getOutputForSigGroups(pcNum,colLabelingNames)
      write2DStringArrayToFile(output, 'pc%02d-condition-groups.txt' % pcNum)

def write2DStringArrayToFile(stringArray, filename, delim='\t'):
  """
  Simple utility function to spew a 2D string array to a tab-delimited
  text file.
  """
  fd = open(filename,'w')
  for row in stringArray:
    for col in row:
      fd.write('%s%s' %(col,delim))
    fd.write('\n')
  fd.close()
